/*
 * Heretic PSP
 * Upstream authors can be found in doc/AUTHORS
 * See docs about original license, the README.txt says that
 * code changes must be GPL'ed
 * PSP Port: Copyright (c) 2005 Gianluigi Tiesi <sherpya@netfarm.it>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <doomdef.h>

int finalestage;                /* 0 = text, 1 = art screen */
int finalecount;

#define TEXTSPEED       3
#define TEXTWAIT        250

char *e1text = E1TEXT;
char *e2text = E2TEXT;
char *e3text = E3TEXT;
char *e4text = E4TEXT;
char *e5text = E5TEXT;
char *finaletext;
char *finaleflat;

int FontABaseLump;

extern boolean automapactive;
extern boolean viewactive;

extern void D_StartTitle(void);

/* F_StartFinale */
void F_StartFinale(void)
{
    gameaction = ga_nothing;
    gamestate = GS_FINALE;
    viewactive = false;
    automapactive = false;
    players[consoleplayer].messageTics = 1;
    players[consoleplayer].message = NULL;

    switch (gameepisode)
    {
        case 1:
            finaleflat = "FLOOR25";
            finaletext = e1text;
            break;
        case 2:
            finaleflat = "FLATHUH1";
            finaletext = e2text;
            break;
        case 3:
            finaleflat = "FLTWAWA2";
            finaletext = e3text;
            break;
        case 4:
            finaleflat = "FLOOR28";
            finaletext = e4text;
            break;
        case 5:
            finaleflat = "FLOOR08";
            finaletext = e5text;
            break;
    }

    finalestage = 0;
    finalecount = 0;
    FontABaseLump = W_GetNumForName("FONTA_S") + 1;

    S_ChangeMusic(mus_cptd, true);
}

boolean F_Responder(event_t * event)
{
    if (event->type != ev_keydown)
        return false;

    if (finalestage == 1 && gameepisode == 2)
    {
        /* we're showing the water pic, make any key kick to demo mode */
        finalestage++;
        memset(screen, 0, screenwidth * screenheight);
        I_SetPalette(W_CacheLumpName("PLAYPAL", PU_CACHE));
        return true;
    }
    return false;
}


/* F_Ticker */
void F_Ticker(void)
{
    finalecount++;
    if (!finalestage && finalecount > strlen(finaletext) * TEXTSPEED + TEXTWAIT)
    {
        finalecount = 0;
        if (!finalestage)
            finalestage = 1;
    }
}


/* F_TextWrite */
void F_TextWrite(void)
{
    byte *src, *dest;
    int x, y;
    int count;
    char *ch;
    int c;
    int cx, cy;
    patch_t *w;

    /* 
     * erase the entire screen to a tiled background
     */
    src = W_CacheLumpName(finaleflat, PU_CACHE);
    dest = screen;
    for (y = 0; y < screenheight; y++)
    {
        for (x = 0; x < screenwidth / 64; x++)
        {
            memcpy(dest, src + ((y & 63) << 6), 64);
            dest += 64;
        }
        if (screenwidth & 63)
        {
            memcpy(dest, src + ((y & 63) << 6), screenwidth & 63);
            dest += (screenwidth & 63);
        }
    }

    /* V_MarkRect (0, 0, SCREENWIDTH, SCREENHEIGHT); */

    /* 
     * draw some of the text onto the screen
     */
    cx = 20;
    cy = 5;
    ch = finaletext;

    count = (finalecount - 10) / TEXTSPEED;
    if (count < 0)
        count = 0;
    for (; count; count--)
    {
        c = *ch++;
        if (!c)
            break;
        if (c == '\n')
        {
            cx = 20;
            cy += 9;
            continue;
        }

        c = toupper(c);
        if (c < 33)
        {
            cx += 5;
            continue;
        }

        w = W_CacheLumpNum(FontABaseLump + c - 33, PU_CACHE);
        if ((cx + w->width) > screenwidth)
            break;

        V_DrawPatch(cx, cy, w);
    }

}


void F_DrawPatchCol(int x, patch_t * patch, int col)
{
    column_t *column;
    byte *source, *dest, *desttop;
    int count;

    column = (column_t *) ((byte *) patch + patch->columnofs[col]);
    desttop = screen + x;

    /* step through the posts in a column */

    while (column->topdelta != 0xff)
    {
        source = (byte *) column + 3;
        dest = desttop + column->topdelta * screenwidth;
        count = column->length;

        while (count--)
        {
            *dest = *source++;
            dest += screenwidth;
        }
        column = (column_t *) ((byte *) column + column->length + 4);
    }
}

  /* 
     ================== = = F_DemonScroll = ================== */

void F_DemonScroll(void)        /* whether it would work ??? */
{
    byte *p1, *p2;
    static int yval = 0;
    static int nextscroll = 0;

    if (finalecount < nextscroll)
        return;

    p1 = W_CacheLumpName("FINAL1", PU_LEVEL);
    p2 = W_CacheLumpName("FINAL2", PU_LEVEL);
    if (finalecount < 70)
    {
        memcpy(screen, p1, screenheight * screenwidth);
        nextscroll = finalecount;
        return;
    }
    if (yval < screenwidth * screenheight)
    {
        memcpy(screen, p2 + screenheight * screenwidth - yval, yval);
        memcpy(screen + yval, p1, screenheight * screenwidth - yval);
        yval += screenwidth;
        nextscroll = finalecount + 3;
    }
    else
        /* else, we'll just sit here and wait, for now */
        memcpy(screen, p2, screenwidth * screenheight);
}

/* ================== = = F_DrawUnderwater = ================== */

void F_DrawUnderwater(void)
{
    static boolean underwawa;
    extern boolean MenuActive;
    extern boolean askforquit;

    switch (finalestage)
    {
        case 1:
            if (!underwawa)
            {
                underwawa = true;
                memset(screen, 0, screenwidth * screenheight);
                I_SetPalette(W_CacheLumpName("E2PAL", PU_CACHE));
                memcpy(screen, W_CacheLumpName("E2END", PU_CACHE),
                       screenwidth * screenheight);
            }
            paused = false;
            MenuActive = false;
            askforquit = false;

            break;
        case 2:
            memcpy(screen, W_CacheLumpName("TITLE", PU_CACHE), screenwidth * screenheight);
    }
}


/* F_Drawer */
void F_Drawer(void)
{
    UpdateState |= I_FULLSCRN;
    if (!finalestage)
        F_TextWrite();
    else
    {
        switch (gameepisode)
        {
            case 1:
                if (shareware)
                    V_DrawRawScreen(W_CacheLumpName("ORDER", PU_CACHE));
                else
                    V_DrawRawScreen(W_CacheLumpName("CREDIT", PU_CACHE));
                break;
            case 2:
                F_DrawUnderwater();
                break;
            case 3:
                F_DemonScroll();
                break;
            case 4: /* Just show credits screen for extended episodes */
            case 5:
                V_DrawRawScreen(W_CacheLumpName("CREDIT", PU_CACHE));
                break;
        }
    }
}
