/*
 * Heretic PSP
 * Upstream authors can be found in doc/AUTHORS
 * See docs about original license, the README.txt says that
 * code changes must be GPL'ed
 * PSP Port: Copyright (c) 2005 Gianluigi Tiesi <sherpya@netfarm.it>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <doomdef.h>

#ifdef PSP
extern void _DisableFPUExceptions();

/* Exit callback */
int exit_callback(void)
{
    pspDebugScreenClear();
    sceKernelExitGame();
    return 0;
}

/* Callback thread */
void CallbackThread(void *arg)
{
    int cbid;
    cbid = sceKernelCreateCallback("Exit Callback", (SceKernelCallbackFunction) exit_callback, NULL);
    sceKernelRegisterExitCallback(cbid);
    sceKernelSleepThreadCB();
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void)
{
    int thid = 0;
    thid = sceKernelCreateThread("update_thread", (SceKernelThreadEntry) CallbackThread, 0x11, 0xFA0, 0, 0);
    if (thid >= 0) sceKernelStartThread(thid, (SceSize) 0, 0);
    return thid;
}
#endif

int main(int argc, char *argv[])
{
    myargc = argc;
    myargv = argv;

#ifdef PSP
    _DisableFPUExceptions();
    SetupCallbacks();
    pspDebugScreenInit();

    sceCtrlSetSamplingCycle(0);
    sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);

    sceKernelDcacheWritebackAll();
    D_DoomMain();
    sceKernelExitGame();
#else
    D_DoomMain();
#endif
    return 0;
}
