/*
 * Heretic PSP
 * Upstream authors can be found in doc/AUTHORS
 * See docs about original license, the README.txt says that
 * code changes must be GPL'ed
 * PSP Port: Copyright (c) 2005 Gianluigi Tiesi <sherpya@netfarm.it>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <doomdef.h>

#ifdef UDP_PROTOCOL
#include <i_udp.h>
#endif

void (*netget) (void);
void (*netsend) (void);

/*
 * I_InitNetwork
 */
void I_InitNetwork(void)
{
    int i;

    doomcom = malloc(sizeof(*doomcom));
    assert(doomcom);
    memset(doomcom, 0, sizeof(*doomcom));
    netbuffer = &doomcom->data;

    /* 
     * shared parameters (IPX/UDP)
     * set up for network
     */
    i = M_CheckParm("-dup");
    if (i && i < myargc - 1)
    {
        doomcom->ticdup = myargv[i + 1][0] - '0';
        if (doomcom->ticdup < 1)
            doomcom->ticdup = 1;
        if (doomcom->ticdup > 9)
            doomcom->ticdup = 9;
    }
    else
        doomcom->ticdup = 1;

    if (M_CheckParm("-extratic"))
        doomcom->extratics = 1;
    else
        doomcom->extratics = 0;
    /* shared parameters end */

#ifdef IPX_PROTOCOL

    /* check, whether IPX network play is wanted */

    i = M_CheckParm("-ipx");
    if (i)
    {
        netsend = IPXPacketSend;
        netget = (void (*)(void)) IPXPacketGet; /* IPXPacketGet() normally
                                                   returns an int */
        netgame = true;
        I_InitIPX();
        return;
    }

#endif

#ifdef UDP_PROTOCOL

    /* 
     * parse UDP/IP network game options,
     *  -net <consoleplayer> <host> <host> ...
     */
    i = M_CheckParm("-net");
    if (i)
    {
        netsend = UDPPacketSend;
        netget = UDPPacketGet;
        netgame = true;
        I_InitUDP();
        return;
    }
#endif

    /* single player game */
    netgame = false;
    doomcom->id = DOOMCOM_ID;
    doomcom->numplayers = doomcom->numnodes = 1;
    doomcom->deathmatch = false;
    doomcom->consoleplayer = 0;

    /* 
     * this was needed, additionally (from old I_InitNetwork())
     * doomcom->ticdup=1;
     * doomcom->extratics=0;
     */
    return;
}


void I_NetCmd(void)
{
    if (doomcom->command == CMD_SEND)
        netsend();
    else if (doomcom->command == CMD_GET)
        netget();
    else
        I_Error("Bad net cmd: %i\n", doomcom->command);
}
