/*
 * Heretic PSP
 * Upstream authors can be found in doc/AUTHORS
 * See docs about original license, the README.txt says that
 * code changes must be GPL'ed
 * PSP Port: Copyright (c) 2005 Gianluigi Tiesi <sherpya@netfarm.it>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <doomdef.h>

long mb_used = 8;
int DisplayTicker = 0;

void I_Tactile(int on, int off, int total)
{
    /* UNUSED. */
    on = off = total = 0;
}

ticcmd_t emptycmd;

ticcmd_t *I_BaseTiccmd(void)
{
    return &emptycmd;
}

int I_GetHeapSize(void)
{
    return mb_used * 1024 * 1024;
}

byte *I_ZoneBase(size_t * size)
{
    byte *dummy;

    *size = mb_used * 1024 * 1024;
    dummy = (byte *) malloc(*size);
    assert(dummy);
    return dummy;
}


/*
 * I_GetTime
 * returns time in 1/70th second tics
 */
int I_GetTime(void)
{
    struct timeval tp;
    struct timezone tzp;
    int newtics;
    static int basetime = 0;

    gettimeofday(&tp, &tzp);
    if (!basetime)
        basetime = tp.tv_sec;
    newtics = (tp.tv_sec - basetime) * TICRATE + tp.tv_usec * TICRATE / 1000000;
    return newtics;
}


/*
 * I_Init
 */
void I_Init(void)
{
    I_InitSound();
    I_InitMusic();
}


/*
 * I_Quit
 */
void I_Quit(void)
{
    isRunning = false;

    D_QuitNetGame();
    I_ShutdownSound();
    I_ShutdownMusic();
    M_SaveDefaults();
    I_ShutdownGraphics();
#ifdef PSP
    sceKernelExitGame();
#endif
}

void I_BeginRead(void) {}
void I_EndRead(void) {}

/* I_Error */
extern boolean demorecording;

#define _ERROR_HDR "Error: "

void I_Error(char *error, ...)
{
    va_list argptr;
    char err_buff[1024];

#ifdef PSP
    sceGuTerm();
    pspDebugScreenInit();
    pspDebugScreenClear();
#endif

    /* put message first into buffer */
    va_start(argptr, error);
    strcpy(err_buff, _ERROR_HDR);
    vsprintf(err_buff + strlen(_ERROR_HDR), error, argptr);
    strcat(err_buff, "\n");
    va_end(argptr);

    /* Shutdown. Here might be other errors. */
    if (demorecording)
        G_CheckDemoStatus();

    D_QuitNetGame();
    I_ShutdownSound();
    I_ShutdownMusic();
    I_ShutdownGraphics();

    printf("%s", err_buff);
    printf("\nYou can send this bug message to: sherpya@netfarm.it\n");

    Stop();
}
