/*
 * Heretic PSP
 * Upstream authors can be found in doc/AUTHORS
 * See docs about original license, the README.txt says that
 * code changes must be GPL'ed
 * SDL graphics-module was taken from the Linux-Hexen port
 * and modified to work with Heretic
 * Jpeg write routing comes from SDLCam
 * Copyright (C) 2002, Raphael Assenat
 *
 * PSP Port: Copyright (c) 2005 Gianluigi Tiesi <sherpya@netfarm.it>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <jpeglib.h>
#include "doomdef.h"

#include <SDL/SDL.h>

#ifdef PSP
static SceCtrlData ctl;
static unsigned long control_bef_ctl = 0;
#endif

/* Public Data */
static int DisplayTicker = 0;
static int ticcount;

extern boolean MenuActive;

SDL_Surface *sdl_screen;

/* I_SetPalette:
 *  Palette source must use 8 bit RGB elements.
 */
void I_SetPalette(byte * palette)
{
    SDL_Color *c;
    SDL_Color *cend;
    SDL_Color cmap[256];

#ifdef PSP
	sceDisplayWaitVblankStart();
#endif

    c = cmap;
    cend = c + 256;
    for (; c != cend; c++)
    {
        c->r = gammatable[usegamma][*palette++];
        c->g = gammatable[usegamma][*palette++];
        c->b = gammatable[usegamma][*palette++];
    }
    SDL_SetColors(sdl_screen, cmap, 0, 256);
}

/* GRAPHICS MODE */
byte *destview;

/* I_Update */
int UpdateState;
extern int screenblocks;

void I_FinishUpdate(void)
{
    int i;
    byte *dest;
    int tics;
    static int lasttic;

    /* blit screen to video */
    if (DisplayTicker)
    {
        dest = (byte *) screen;

        tics = ticcount - lasttic;
        lasttic = ticcount;
        if (tics > 20)
            tics = 20;

        for (i = 0; i < tics; i++)
        {
            *dest = 0xff;
            dest += 2;
        }

        for (i = tics; i < 20; i++)
        {
            *dest = 0x00;
            dest += 2;
        }
    }
    SDL_UpdateRect(sdl_screen, 0, 0, screenwidth, screenheight);
}

void InitGraphLib(void)
{
#if defined(__DOSOUND__) && !defined(PSP)
    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0)
#else
    if (SDL_Init(SDL_INIT_VIDEO) < 0)
#endif
        I_Error("SDL_Init Failed\n");
}

/* I_InitGraphics */
void I_InitGraphics(void)
{
    int flags;
    ticcount = (SDL_GetTicks() * 35) / 1000;

    sdl_screen = SDL_SetVideoMode(screenwidth, screenheight, 8, SDL_HWSURFACE);

    if (!sdl_screen)
        I_Error("[SDLDrv] Couldn't set video mode %dx%d: %s\n", screenwidth, screenheight, SDL_GetError());

    if (SDL_MUSTLOCK(sdl_screen))
        I_Error("[SDLDrv] SDL_MUSTLOCK\n");

    SDL_ShowCursor(0);
    SDL_WM_SetCaption("PSPHeretic v1.03", "Heretic");

	I_SetPalette(W_CacheLumpName("PLAYPAL", PU_CACHE));

    screen = malloc(screenheight * screenwidth);
    if (!screen)
        I_Error("Couldn't allocate space for screenmemory !\n");
    sdl_screen->pixels = screen;
}

/* I_ShutdownGraphics */
void I_ShutdownGraphics(void)
{
    SDL_Quit();
}

int write_file_jpeg (char *file_name, unsigned char *data, int width, int height)
{
    struct jpeg_compress_struct jpeg;
    struct jpeg_error_mgr jpeg_err;
    int i;
    unsigned char *line;
    FILE *fp = fopen(file_name,"wb");
    if (!fp) return -1;

    jpeg.err = jpeg_std_error(&jpeg_err);
    jpeg_create_compress(&jpeg);
    jpeg_stdio_dest(&jpeg, fp);
    jpeg.in_color_space = JCS_RGB;
    jpeg.image_width = width;
    jpeg.image_height = height;
    jpeg.input_components = 3;
    jpeg.dct_method = JDCT_IFAST; /* JDCT_FLOAT */
    jpeg_set_defaults(&jpeg);
    jpeg_set_quality(&jpeg, 85, TRUE);
    jpeg_start_compress(&jpeg, TRUE);
    for (i = 0, line = data; i < height; i++, line += width*3)
        jpeg_write_scanlines(&jpeg, &line, 1);
    jpeg_finish_compress(&jpeg);
    jpeg_destroy_compress(&jpeg);
    fclose(fp);
    return 0;
}

SDL_Surface *get_ImageSurface(SDL_Surface *sc, int w, int h)
{
    SDL_Surface *tmp_surface;
    SDL_Rect source_rect;

    tmp_surface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 24, 0, 0, 0, 255);
    source_rect.w = w;
    source_rect.h = h;
    source_rect.x = 0;
    source_rect.y = 0;
    SDL_BlitSurface(sc, &source_rect, tmp_surface, NULL);
    return tmp_surface;
}

void SDL_ScreenShot(void)
{
    int i;
    char lbmname[32];
    unsigned char *data;
    unsigned char tmp;
    SDL_Surface *sface = get_ImageSurface(sdl_screen, screenwidth, screenheight);

    if (SDL_MUSTLOCK(sdl_screen))
    {
        if (SDL_LockSurface(sdl_screen) < 0)
            return;
    }

    /* swap Red and blue */
    data = (unsigned char *) sface->pixels;
    for (i = 0; i < sface->w * sface->h * 3; i += 3)
    {
        tmp = data[i];
        data[i] = data[i+2];
        data[i+2] = tmp;
    }

    if (SDL_MUSTLOCK (sdl_screen))
        SDL_UnlockSurface(sdl_screen);

    strcpy(lbmname, "ms0:/psp/photo/heretic00.jpg");

    for (i = 0; i <= 99; i++)
    {
        lbmname[22] = i / 10 + '0';
        lbmname[23] = i % 10 + '0';
        if (access(lbmname, 0) == -1)
            break; /* file doesn't exist */
    }
    if (i == 100)
        I_Error("SDL_ScreenShot: Couldn't create a jpeg");

    write_file_jpeg(lbmname, (unsigned char*) sface->pixels, sface->w, sface->h);
}

#ifdef PSP
unsigned long Read_Key(void)
{
    sceCtrlReadBufferPositive(&ctl, 1);

    if (ctl.Buttons == control_bef_ctl)
        return 0;
    control_bef_ctl  = ctl.Buttons;
    return control_bef_ctl;
}
#endif

#define DO_KEY_EVENT(pspkey, gamekey) \
{ \
    if (key & pspkey) \
    { \
        kbevent.type = ev_keydown; \
        kbevent.data1 = gamekey; \
        D_PostEvent(&kbevent); \
    } \
    else \
    { \
        kbevent.type = ev_keyup; \
        kbevent.data1 = gamekey; \
        D_PostEvent(&kbevent); \
    } \
}

#ifdef PSP
void I_GetEvent(void)
{
    static event_t event;
    static event_t kbevent;
 
    unsigned long key;

    event.type = ev_joystick;
    event.data1 = 0;
    event.data2 = 0;
    event.data3 = 0;

    key = Read_Key();
 
    if (ctl.Ly >= 0xD0) event.data3 = 1;   /* DOWN */
    if (ctl.Ly <= 0x10) event.data3 = -1;  /* UP */
    if (ctl.Lx <= 0x10) event.data2 = -1;  /* LEFT */
    if (ctl.Lx >= 0xD0) event.data2 = 1;   /* RIGHT */

    D_PostEvent(&event);

    if (MenuActive)
    {
        DO_KEY_EVENT(PSP_CTRL_UP, KEY_UPARROW);
        DO_KEY_EVENT(PSP_CTRL_DOWN, KEY_DOWNARROW);
        DO_KEY_EVENT(PSP_CTRL_LEFT, KEY_LEFTARROW);
        DO_KEY_EVENT(PSP_CTRL_RIGHT, KEY_RIGHTARROW);
        DO_KEY_EVENT(PSP_CTRL_CROSS, KEY_ENTER);
        DO_KEY_EVENT(PSP_CTRL_CIRCLE, KEY_ESCAPE);
        return;
    }

    if (key & PSP_CTRL_UP)
        SDL_ScreenShot();

    DO_KEY_EVENT(PSP_CTRL_LEFT, 'k');
    DO_KEY_EVENT(PSP_CTRL_RIGHT, 'l');
    DO_KEY_EVENT(PSP_CTRL_DOWN, ' ');
    DO_KEY_EVENT(PSP_CTRL_START, KEY_ENTER);
    DO_KEY_EVENT(PSP_CTRL_SELECT, KEY_ESCAPE);
    DO_KEY_EVENT(PSP_CTRL_CROSS, KEY_RCTRL);
    DO_KEY_EVENT(PSP_CTRL_TRIANGLE, KEY_TAB);
    DO_KEY_EVENT(PSP_CTRL_SQUARE, ',');
    DO_KEY_EVENT(PSP_CTRL_CIRCLE, '.')
    DO_KEY_EVENT(PSP_CTRL_LTRIGGER, KEY_MINUS);
    DO_KEY_EVENT(PSP_CTRL_RTRIGGER, KEY_EQUALS);
}
#else
int xlatekey(SDL_keysym *key)
{
    switch(key->sym)
    {
        case SDLK_LEFT:      return KEY_LEFTARROW;
        case SDLK_RIGHT:     return KEY_RIGHTARROW;
        case SDLK_DOWN:      return KEY_DOWNARROW;
        case SDLK_UP:        return KEY_UPARROW;
        case SDLK_ESCAPE:    return KEY_ESCAPE;
        case SDLK_RETURN:    return KEY_ENTER;
        case SDLK_F1:        return KEY_F1;
        case SDLK_F2:        return KEY_F2;
        case SDLK_F3:        return KEY_F3;
        case SDLK_F4:        return KEY_F4;
        case SDLK_F5:        return KEY_F5;
        case SDLK_F6:        return KEY_F6;
        case SDLK_F7:        return KEY_F7;
        case SDLK_F8:        return KEY_F8;
        case SDLK_F9:        return KEY_F9;
        case SDLK_F10:       return KEY_F10;
        case SDLK_F11:       return KEY_F11;
        case SDLK_F12:       return KEY_F12;
        case SDLK_INSERT:    return KEY_INSERT;
        case SDLK_DELETE:    return KEY_DELETE;
        case SDLK_PAGEUP:    return KEY_PAGEUP;
        case SDLK_PAGEDOWN:  return KEY_PAGEDOWN;
        case SDLK_HOME:      return KEY_HOME;
        case SDLK_END:       return KEY_END;
        case SDLK_BACKSPACE: return KEY_BACKSPACE;
        case SDLK_PAUSE:     return KEY_PAUSE;
        case SDLK_EQUALS:    return KEY_EQUALS;
        case SDLK_KP_MINUS:
        case SDLK_MINUS:     return KEY_MINUS;
        case SDLK_KP_PLUS:
        case SDLK_PLUS:      return KEY_EQUALS;
        case SDLK_LSHIFT:
        case SDLK_RSHIFT:    return KEY_RSHIFT;
        case SDLK_LALT:
        case SDLK_LMETA:
        case SDLK_RALT:
        case SDLK_RMETA:     return KEY_RALT;
        default:
            return key->sym;
    }
    return key->sym;
}

void I_GetEvent(SDL_Event *Event)
{
    event_t event;

    switch (Event->type)
    {
        case SDL_KEYDOWN:
            event.type = ev_keydown;
            event.data1 = xlatekey(&Event->key.keysym);
            D_PostEvent(&event);
            break;
        case SDL_KEYUP:
            event.type = ev_keyup;
            event.data1 = xlatekey(&Event->key.keysym);
            D_PostEvent(&event);
            break;
        case SDL_QUIT:
            I_Quit();
    }
}
#endif

/* I_StartTic */
void I_StartTic(void)
{
#ifdef PSP
    I_GetEvent();
#else
    SDL_Event Event;
    while (SDL_PollEvent(&Event))
        I_GetEvent(&Event);
#endif
}

/* I_StartFrame */
void I_StartFrame(void) {}
